
__version__ = "0.2.5"
__author__ = "<符玄(KOKOMI12345)>"
__email__ = "3072252442@qq.com"
__license__ = "MIT"
__copyright__ = "Copyright 2024 KOKOMI12345"
__python_version__ = "3.12.6"
__project_name__ = "FuXLogger"
__github_url__ = "https://github.com/KOKOMI12345/FuXLogger"
